<?php

namespace Mavabc\Solution\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Mavabc\Solution\Http\Requests\SolutionsRequest;
use Mavabc\Solution\Interfaces\SolutionsRepositoryInterface;
use Mavabc\Solution\Models\Solutions;

/**
 * Resource controller class for solutions.
 */
class SolutionsResourceController extends BaseController
{

    /**
     * Initialize solutions resource controller.
     *
     * @param type SolutionsRepositoryInterface $solutions
     *
     * @return null
     */
    public function __construct(SolutionsRepositoryInterface $solutions)
    {
        parent::__construct();
        $this->repository = $solutions;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Mavabc\Solution\Repositories\Criteria\SolutionsResourceCriteria::class);
    }

    /**
     * Display a list of solutions.
     *
     * @return Response
     */
    public function index(SolutionsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Mavabc\Solution\Repositories\Presenter\SolutionsPresenter::class)
                ->$function();
        }

        $solutions = $this->repository->paginate();

        return $this->response->title(trans('solution::solutions.names'))
            ->view('solution::solutions.index', true)
            ->data(compact('solutions'))
            ->output();
    }

    /**
     * Display solutions.
     *
     * @param Request $request
     * @param Model   $solutions
     *
     * @return Response
     */
    public function show(SolutionsRequest $request, Solutions $solutions)
    {

        if ($solutions->exists) {
            $view = 'solution::solutions.show';
        } else {
            $view = 'solution::solutions.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('solution::solutions.name'))
            ->data(compact('solutions'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new solutions.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SolutionsRequest $request)
    {

        $solutions = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('solution::solutions.name')) 
            ->view('solution::solutions.create', true) 
            ->data(compact('solutions'))
            ->output();
    }

    /**
     * Create new solutions.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SolutionsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $solutions                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('solution::solutions.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('solution/solutions/' . $solutions->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/solution/solutions'))
                ->redirect();
        }

    }

    /**
     * Show solutions for editing.
     *
     * @param Request $request
     * @param Model   $solutions
     *
     * @return Response
     */
    public function edit(SolutionsRequest $request, Solutions $solutions)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('solution::solutions.name'))
            ->view('solution::solutions.edit', true)
            ->data(compact('solutions'))
            ->output();
    }

    /**
     * Update the solutions.
     *
     * @param Request $request
     * @param Model   $solutions
     *
     * @return Response
     */
    public function update(SolutionsRequest $request, Solutions $solutions)
    {
        try {
            $attributes = $request->all();

            $solutions->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('solution::solutions.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('solution/solutions/' . $solutions->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('solution/solutions/' . $solutions->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the solutions.
     *
     * @param Model   $solutions
     *
     * @return Response
     */
    public function destroy(SolutionsRequest $request, Solutions $solutions)
    {
        try {

            $solutions->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('solution::solutions.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('solution/solutions/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('solution/solutions/' . $solutions->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple solutions.
     *
     * @param Model   $solutions
     *
     * @return Response
     */
    public function delete(SolutionsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('solution::solutions.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('solution/solutions'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/solution/solutions'))
                ->redirect();
        }

    }

    /**
     * Restore deleted solutions.
     *
     * @param Model   $solutions
     *
     * @return Response
     */
    public function restore(SolutionsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('solution::solutions.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/solution/solutions'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/solution/solutions/'))
                ->redirect();
        }

    }

}
